require 'yaml'
require 'fileutils'
require 'logger'
require 'dotenv/load'

module Maschine
    class Generator
        @@scaffoldDirs = %w(
            src src/_includes src/_includes/sections src/_includes/snippets src/_pages src/_data src/_layouts
            src/_plugins src/_posts src/_sass src/_config src/_locales assets webpack
        ).freeze

        @@rootCopyFiles = %w(
            templates/Gemfile templates/package.json templates/webpack.config.js
        ).freeze

        @@erbFiles = %( templates/_config.yml.erb )

        def initialize(project_path=nil)
            @gemPath = File.expand_path File.dirname(__FILE__) + "/../.."
            configFile = File.join(@gemPath, 'config.yaml')
             
            raise "config.yaml does not exists" if ! File.exists?(configFile)
            @config = YAML.load_file(configFile)            
            @projectPath = ENV['MASCHINE_PROJECT_DIRECTORY']            
            @logger = Logger.new(STDOUT)
            @logger.level = Logger::DEBUG
            # @logger.debug @projectPath
            # @logger.debug @gemPath
            @apikey = ENV['MASCHINE_API_KEY']
        end

        def apikey
            return @apikey
        end

        def createDirs            
            @@scaffoldDirs.each do |directory|
                dir = File.join(@projectPath, directory)                
                FileUtils.mkdir_p(File.join(@projectPath, directory)) if !Dir.exists?(dir)
            end
        end

        def dirs
            dir = Array.new
            @@scaffoldDirs.each do |directory|                
                d = File.join(@projectPath, directory)                
                dir << d 
            end
            return dir
        end

        def erb
            @erb ||= ERBRenderer.new(self)
        end

        def copyTheme
            import = @config['maschine']['theme']['import']
            importDir =  File.join(@projectPath, import['directory'])
            import['process'].each do |directory|
                from = File.join(importDir, directory['directory']['from'])
                to = File.join(@projectPath, directory['directory']['to'])
                FileUtils.cp_r from, to
            end            
        end

        def rootFiles
            files = Array.new
            @@rootCopyFiles.each do |file| 
                files.push(File.join(@gemPath, file))                            
            end
            return files
        end
        
        def copyRootFiles
            rootFiles.each do |file|
                if File.exists?(file)
                    FileUtils.cp(file, @projectPath)
                else
                    @logger.error(file)    
                end    
            end
        end

        def processConfigFile
        end

        def execute
            @logger.debug "create directories"
            self.createDirs
            # TODO: hole das theme vom server
            @logger.debug "copy theme"
            self.copyTheme
            @logger.debug "copy root files"
        end
    end
end